@@ -5,13 +5,10 @@ import os
 import sys
 from datetime import datetime
 
-from virtualenv.config.cli.parser import VirtualEnvOptions
-from virtualenv.util.six import ensure_text
-
 
 def run(args=None, options=None):
     start = datetime.now()
-    from virtualenv.error import ProcessCallFailed
+    from virtualenv.util.error import ProcessCallFailed
     from virtualenv.run import cli_run
 
     if args is None:
@@ -32,6 +29,8 @@ class LogSession(object):
         self.start = start
 
     def __str__(self):
+        from virtualenv.util.six import ensure_text
+
         spec = self.session.creator.interpreter.spec
         elapsed = (datetime.now() - self.start).total_seconds() * 1000
         lines = [
@@ -39,24 +38,41 @@ class LogSession(object):
             "  creator {}".format(ensure_text(str(self.session.creator))),
         ]
         if self.session.seeder.enabled:
-            lines += ("  seeder {}".format(ensure_text(str(self.session.seeder)),),)
+            lines += (
+                "  seeder {}".format(ensure_text(str(self.session.seeder))),
+                "    added seed packages: {}".format(
+                    ", ".join(
+                        sorted(
+                            "==".join(i.stem.split("-"))
+                            for i in self.session.creator.purelib.iterdir()
+                            if i.suffix == ".dist-info"
+                        ),
+                    ),
+                ),
+            )
         if self.session.activators:
             lines.append("  activators {}".format(",".join(i.__class__.__name__ for i in self.session.activators)))
         return os.linesep.join(lines)
 
 
 def run_with_catch(args=None):
+    from virtualenv.config.cli.parser import VirtualEnvOptions
+
     options = VirtualEnvOptions()
     try:
         run(args, options)
-    except (KeyboardInterrupt, Exception) as exception:
-        if getattr(options, "with_traceback", False):
+    except (KeyboardInterrupt, SystemExit, Exception) as exception:
+        try:
+            if getattr(options, "with_traceback", False):
+                raise
+            else:
+                if not (isinstance(exception, SystemExit) and exception.code == 0):
+                    logging.error("%s: %s", type(exception).__name__, exception)
+                code = exception.code if isinstance(exception, SystemExit) else 1
+                sys.exit(code)
+        finally:
             logging.shutdown()  # force flush of log messages before the trace is printed
-            raise
-        else:
-            logging.error("%s: %s", type(exception).__name__, exception)
-            sys.exit(1)
 
 
-if __name__ == "__main__":
-    run_with_catch()
+if __name__ == "__main__":  # pragma: no cov
+    run_with_catch()  # pragma: no cov
