@@ -188,11 +188,11 @@ COVERAGE_RUN = os.environ.get(str(COV_ENV_VAR))
 
 
 @pytest.fixture(autouse=True)
-def coverage_env(monkeypatch, link):
+def coverage_env(monkeypatch, link, request):
     """
     Enable coverage report collection on the created virtual environments by injecting the coverage project
     """
-    if COVERAGE_RUN:
+    if COVERAGE_RUN and "no_coverage" not in request.fixturenames:
         # we inject right after creation, we cannot collect coverage on site.py - used for helper scripts, such as debug
         from virtualenv import run
 
@@ -230,6 +230,12 @@ def coverage_env(monkeypatch, link):
         yield finish
 
 
+# no_coverage tells coverage_env to disable coverage injection for no_coverage user.
+@pytest.fixture
+def no_coverage():
+    pass
+
+
 class EnableCoverage(object):
     _COV_FILE = Path(coverage.__file__)
     _ROOT_COV_FILES_AND_FOLDERS = [i for i in _COV_FILE.parents[1].iterdir() if i.name.startswith("coverage")]
